@@ -13,7 +13,9 @@ from flask_login import login_required, login_user, logout_user, current_user
 from flask_login.config import EXEMPT_METHODS
 
 
-from .helpers import alias, get_devices_with_keys_by_type, hash_password, get_loglevel, get_version_info, run_shell, set_loglevel, verify_password
+from .helpers import (alias, get_devices_with_keys_by_type, hash_password, 
+                      get_loglevel, get_version_info, run_shell, set_loglevel, 
+                      verify_password, bcur2base64)
 from .specter import Specter
 from .specter_error import SpecterError
 from .wallet_manager import purposes
@@ -35,7 +37,6 @@ def selfcheck():
     """check status before every request"""
     if app.config.get('LOGIN_DISABLED'):
         app.login('admin')
-    app.specter.check()
 
 ########## template injections #############
 @app.context_processor
@@ -55,6 +56,10 @@ def combine(wallet_alias):
     if request.method == 'POST': # FIXME: ugly...
         psbt0 = request.form.get('psbt0') # request.args.get('psbt0')
         psbt1 = request.form.get('psbt1') # request.args.get('psbt1')
+        if "UR:BYTES/" in psbt0:
+            psbt0 = bcur2base64(psbt0)
+        if "UR:BYTES/" in psbt1:
+            psbt1 = bcur2base64(psbt1)
         txid = request.form.get('txid')
 
         try:
@@ -98,6 +103,7 @@ def broadcast(wallet_alias):
 @login_required
 def index():
     notify_upgrade()
+    app.specter.check()
     if len(app.specter.wallet_manager.wallets) > 0:
         return redirect("/wallets/%s" % app.specter.wallet_manager.wallets[app.specter.wallet_manager.wallets_names[0]].alias)
 
@@ -111,6 +117,7 @@ def index():
 @app.route('/login', methods=['GET', 'POST'])
 def login():
     ''' login '''
+    app.specter.check()
     if request.method == 'POST':
         if app.specter.config['auth'] == 'none':
             app.login('admin')
@@ -158,6 +165,7 @@ def redirect_login(request):
 @app.route('/register', methods=['GET', 'POST'])
 def register():
     ''' register '''
+    app.specter.check()
     if request.method == 'POST':
         username = request.form['username']
         password = hash_password(request.form['password'])
@@ -203,6 +211,7 @@ def settings():
 @login_required
 def hwi_settings():
     current_version = notify_upgrade()
+    app.specter.check()
     return render_template(
         "settings/hwi_settings.jinja",
         specter=app.specter,
@@ -214,6 +223,7 @@ def hwi_settings():
 @login_required
 def general_settings():
     current_version = notify_upgrade()
+    app.specter.check()
     explorer = app.specter.explorer
     hwi_bridge_url = app.specter.hwi_bridge_url
     loglevel = get_loglevel(app)
@@ -245,6 +255,7 @@ def general_settings():
 @login_required
 def bitcoin_core_settings():
     current_version = notify_upgrade()
+    app.specter.check()
     if not current_user.is_admin:
         flash('Only an admin is allowed to access this page.', 'error')
         return redirect("/")
@@ -309,20 +320,24 @@ def bitcoin_core_settings():
 @login_required
 def auth_settings():
     current_version = notify_upgrade()
+    app.specter.check()
     auth = app.specter.config['auth']
     new_otp = -1
+    users = None
+    if current_user.is_admin and auth == "usernamepassword":
+        users = [user for user in User.get_all_users(app.specter) if not user.is_admin]
     if request.method == 'POST':
         action = request.form['action']
-        if 'specter_username' in request.form:
+
+        if action == "save":
+            if 'specter_username' in request.form:
                 specter_username = request.form['specter_username']
                 specter_password = request.form['specter_password']
-        else:
-            specter_username = None
-            specter_password = None
-        if current_user.is_admin:
-            auth = request.form['auth']
-            
-        if action == "save":
+            else:
+                specter_username = None
+                specter_password = None
+            if current_user.is_admin:
+                auth = request.form['auth']
             if specter_username:
                 if current_user.username != specter_username:
                     if User.get_user_by_name(app.specter, specter_username):
@@ -331,6 +346,7 @@ def auth_settings():
                             "settings/auth_settings.jinja",
                             auth=auth,
                             new_otp=new_otp,
+                            users=users,
                             specter=app.specter,
                             current_version=current_version,
                             rand=rand
@@ -354,10 +370,23 @@ def auth_settings():
                 flash('New user link generated successfully: {}register?otp={}'.format(request.url_root, new_otp), 'info')
             else:
                 flash('Error: Only the admin account can issue new registration links.', 'error')
+        elif action == "deleteuser":
+            delete_user = request.form['deleteuser']
+            if current_user.is_admin:
+                user = User.get_user(app.specter, delete_user)
+                if user:
+                    user.delete(app.specter)
+                    users = [user for user in User.get_all_users(app.specter) if not user.is_admin]
+                    flash('User {} was deleted successfully'.format(user.username), 'info')
+                else:
+                    flash('Error: failed to delete user, invalid user ID was given', 'error')
+            else:
+                flash('Error: Only the admin account can delete users', 'error')
     return render_template(
         "settings/auth_settings.jinja",
         auth=auth,
         new_otp=new_otp,
+        users=users,
         specter=app.specter,
         current_version=current_version,
         rand=rand
@@ -369,6 +398,7 @@ def auth_settings():
 @app.route('/new_wallet/')
 @login_required
 def new_wallet_type():
+    app.specter.check()
     err = None
     if app.specter.chain is None:
         err = "Configure Bitcoin Core to create wallets"
@@ -383,6 +413,7 @@ def new_wallet(wallet_type):
     if wallet_type not in wallet_types:
         err = "Unknown wallet type requested"
         return render_template("base.jinja", specter=app.specter, rand=rand)
+    app.specter.check()
     name = wallet_type.title()
     wallet_name = name
     i = 2
@@ -523,6 +554,7 @@ def new_wallet(wallet_type):
 @app.route('/wallets/<wallet_alias>/')
 @login_required
 def wallet(wallet_alias):
+    app.specter.check()
     try:
         wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
     except SpecterError as se:
@@ -536,6 +568,7 @@ def wallet(wallet_alias):
 @app.route('/wallets/<wallet_alias>/tx/')
 @login_required
 def wallet_tx(wallet_alias):
+    app.specter.check()
     try:
         wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
     except SpecterError as se:
@@ -546,6 +579,7 @@ def wallet_tx(wallet_alias):
 @app.route('/wallets/<wallet_alias>/addresses/', methods=['GET', 'POST'])
 @login_required
 def wallet_addresses(wallet_alias):
+    app.specter.check()
     try:
         wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
     except SpecterError as se:
@@ -569,6 +603,7 @@ def wallet_addresses(wallet_alias):
 @app.route('/wallets/<wallet_alias>/receive/', methods=['GET', 'POST'])
 @login_required
 def wallet_receive(wallet_alias):
+    app.specter.check()
     try:
         wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
     except SpecterError as se:
@@ -594,6 +629,7 @@ def fees(blocks):
 @app.route('/wallets/<wallet_alias>/send/new', methods=['GET', 'POST'])
 @login_required
 def wallet_send(wallet_alias):
+    app.specter.check()
     try:
         wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
     except SpecterError as se:
@@ -644,7 +680,7 @@ def wallet_send(wallet_alias):
                 b64psbt = request.form["rawpsbt"]
                 psbt = wallet.importpsbt(b64psbt)
             except Exception as e:
-                flash("Could not import PSBT: %s" % e)
+                flash("Could not import PSBT: %s" % e, "error")
                 return redirect(url_for('wallet_importpsbt', wallet_alias=wallet_alias))
             return render_template("wallet/send/sign/wallet_send_sign_psbt.jinja", psbt=psbt, label=label, 
                                                 wallet_alias=wallet_alias, wallet=wallet, 
@@ -658,7 +694,7 @@ def wallet_send(wallet_alias):
             try:
                 wallet.delete_pending_psbt(ast.literal_eval(request.form["pending_psbt"])["tx"]["txid"])
             except Exception as e:
-                flash("Could not delete Pending PSBT!")
+                flash("Could not delete Pending PSBT!", "error")
     return render_template("wallet/send/new/wallet_send.jinja", psbt=psbt, label=label, 
                                                 wallet_alias=wallet_alias, wallet=wallet, 
                                                 specter=app.specter, rand=rand, error=err)
@@ -666,6 +702,7 @@ def wallet_send(wallet_alias):
 @app.route('/wallets/<wallet_alias>/send/import')
 @login_required
 def wallet_importpsbt(wallet_alias):
+    app.specter.check()
     try:
         wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
     except SpecterError as se:
@@ -679,6 +716,7 @@ def wallet_importpsbt(wallet_alias):
 @app.route('/wallets/<wallet_alias>/send/pending/', methods=['GET', 'POST'])
 @login_required
 def wallet_sendpending(wallet_alias):
+    app.specter.check()
     try:
         wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
     except SpecterError as se:
@@ -691,7 +729,7 @@ def wallet_sendpending(wallet_alias):
                 wallet.delete_pending_psbt(ast.literal_eval(request.form["pending_psbt"])["tx"]["txid"])
             except Exception as e:
                 app.logger.error("Could not delete Pending PSBT: %s" % e)
-                flash("Could not delete Pending PSBT!")
+                flash("Could not delete Pending PSBT!", "error")
     pending_psbts = wallet.pending_psbts
     return render_template("wallet/send/pending/wallet_sendpending.jinja", pending_psbts=pending_psbts,
                                                 wallet_alias=wallet_alias, wallet=wallet, 
@@ -701,6 +739,7 @@ def wallet_sendpending(wallet_alias):
 @app.route('/wallets/<wallet_alias>/settings/', methods=['GET','POST'])
 @login_required
 def wallet_settings(wallet_alias):
+    app.specter.check()
     error = None
     try:
         wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
@@ -765,6 +804,7 @@ def wallet_settings(wallet_alias):
 @app.route('/new_device/', methods=['GET', 'POST'])
 @login_required
 def new_device():
+    app.specter.check()
     err = None
     device_type = "other"
     device_name = ""
@@ -790,6 +830,7 @@ def new_device():
 @app.route('/devices/<device_alias>/', methods=['GET', 'POST'])
 @login_required
 def device(device_alias):
+    app.specter.check()
     err = None
     try:
         device = app.specter.device_manager.get_by_alias(device_alias)
@@ -804,24 +845,30 @@ def device(device_alias):
             else:
                 app.specter.device_manager.remove_device(device)
                 return redirect("/")
-        if action == "delete_key":
+        elif action == "delete_key":
             key = request.form['key']
             device.remove_key(Key.from_json({ 'original': key }))
-        if action == "add_keys":
-            return render_template("device/new_device.jinja", device=device, specter=app.specter, rand=rand)
-        if action == "morekeys":
+        elif action == "add_keys":
+            return render_template("device/new_device.jinja", 
+                    device=device, device_alias=device_alias, specter=app.specter, rand=rand)
+        elif action == "morekeys":
             # refactor to fn
             xpubs = request.form['xpubs']
             keys, failed = Key.parse_xpubs(xpubs)
             err = None
             if len(failed) > 0:
                 err = "Failed to parse these xpubs:\n" + "\n".join(failed)
-                return render_template("device/new_device.jinja", device=device, xpubs=xpubs, error=err, specter=app.specter, rand=rand)
+                return render_template("device/new_device.jinja", 
+                        device=device, device_alias=device_alias, xpubs=xpubs, error=err, specter=app.specter, rand=rand)
             if err is None:
                 device.add_keys(keys)
+        elif action == "settype":
+            device_type = request.form['device_type']
+            device.set_type(device_type)
     device = copy.deepcopy(device)
     device.keys.sort(key=lambda k: k.metadata["chain"] + k.metadata["purpose"], reverse=True)
-    return render_template("device/device.jinja", device=device, purposes=purposes, wallets=wallets, error=err, specter=app.specter, rand=rand)
+    return render_template("device/device.jinja", 
+            device=device, device_alias=device_alias, purposes=purposes, wallets=wallets, error=err, specter=app.specter, rand=rand)
 
 
 
@@ -845,5 +892,5 @@ def notify_upgrade():
     version_info["current"], version_info["latest"], version_info["upgrade"] = get_version_info()
     app.logger.info("Upgrade? {}".format(version_info["upgrade"]))
     if version_info["upgrade"]:
-        flash("There is a new version available. Consider strongly to upgrade to the new version {} with \"pip3 install cryptoadvance.specter --upgrade\"".format(version_info["latest"]))
+        flash("There is a new version available. Consider strongly to upgrade to the new version {} with \"pip3 install cryptoadvance.specter --upgrade\"".format(version_info["latest"]), "info")
     return version_info["current"]
