@@ -5,7 +5,8 @@ from .devices.trezor import Trezor
 from .devices.ledger import Ledger
 from .devices.keepkey import Keepkey
 from .devices.specter import Specter
-from .helpers import alias, load_jsons
+from .devices.cobo import Cobo
+from .helpers import alias, load_jsons, fslock
 
 
 logger = logging.getLogger(__name__)
@@ -15,7 +16,8 @@ device_classes = {
     'trezor': Trezor,
     'keepkey': Keepkey,
     'ledger': Ledger,
-    'specter': Specter
+    'specter': Specter,
+    'cobo': Cobo,
 }
 
 def get_device_class(device_type):
@@ -39,16 +41,17 @@ class DeviceManager:
             # creating folders if they don't exist
             if not os.path.isdir(data_folder):
                 os.mkdir(data_folder)
-        self.devices = {}
+        devices = {}
         devices_files = load_jsons(self.data_folder, key="name")
         for device_alias in devices_files:
             fullpath = os.path.join(self.data_folder, "%s.json" % device_alias)
-            self.devices[devices_files[device_alias]["name"]] = get_device_class(devices_files[device_alias]["type"]).from_json(
+            devices[devices_files[device_alias]["name"]] = get_device_class(devices_files[device_alias]["type"]).from_json(
                 devices_files[device_alias],
                 self,
                 default_alias=device_alias,
                 default_fullpath=fullpath
             )
+        self.devices = devices
 
     @property
     def devices_names(self):
@@ -69,8 +72,9 @@ class DeviceManager:
                 non_dup_keys.append(key)
         keys = non_dup_keys
         device = get_device_class(device_type)(name, device_alias, device_type, keys, fullpath, self)
-        with open(fullpath, "w") as file:
-            file.write(json.dumps(device.json, indent=4))
+        with fslock:
+            with open(fullpath, "w") as file:
+                file.write(json.dumps(device.json, indent=4))
 
         self.update() # reload files
         return device
